@@ -98,6 +98,45 @@ _xfconf_gtype_from_string(const gchar *type)
     return G_TYPE_INVALID;
 }
 
+G_CONST_RETURN gchar *
+_xfconf_string_from_gtype(GType gtype)
+{
+    switch(gtype) {
+        case G_TYPE_STRING:
+            return "string";
+        case G_TYPE_UCHAR:
+            return "uchar";
+        case G_TYPE_CHAR:
+            return "char";
+        case G_TYPE_UINT:
+            return "uint";
+        case G_TYPE_INT:
+            return "int";
+        case G_TYPE_UINT64:
+            return "uint64";
+        case G_TYPE_INT64:
+            return "int64";
+        case G_TYPE_FLOAT:
+            return "float";
+        case G_TYPE_DOUBLE:
+            return "double";
+        case G_TYPE_BOOLEAN:
+            return "bool";
+        default:
+            if(gtype == XFCONF_TYPE_UINT16)
+                return "uint16";
+            else if(gtype == XFCONF_TYPE_INT16)
+                return "int16";
+            else if(gtype == XFCONF_TYPE_G_VALUE_ARRAY)
+                return "array";
+            break;
+    }
+
+    g_warning("GType %s doesn't map to an Xfconf type",
+              g_type_name(gtype));
+    return NULL;
+}
+
 gboolean
 _xfconf_gvalue_from_string(GValue *value,
                            const gchar *str)
@@ -240,8 +279,7 @@ _xfconf_string_from_gvalue(GValue *val)
         case G_TYPE_DOUBLE:
             return g_strdup_printf("%f", g_value_get_double(val));
         case G_TYPE_BOOLEAN:
-            return g_strdup(g_value_get_boolean(val) ? _("true")
-                                                     : _("false"));
+            return g_strdup(g_value_get_boolean(val) ? "true" : "false");
         default:
             if(G_VALUE_TYPE(val) == XFCONF_TYPE_UINT16) {
                 return g_strdup_printf("%u",
@@ -257,6 +295,52 @@ _xfconf_string_from_gvalue(GValue *val)
     return NULL;
 }
 
+gboolean
+_xfconf_gvalue_is_equal(const GValue *value1,
+                        const GValue *value2)
+{
+    if(G_UNLIKELY(!value1 && !value2))
+        return TRUE;
+    if(G_UNLIKELY(!value1 || !value2))
+        return FALSE;
+    if(G_VALUE_TYPE(value1) != G_VALUE_TYPE(value2))
+        return FALSE;
+    if(G_VALUE_TYPE(value1) == G_TYPE_INVALID
+       || G_VALUE_TYPE(value1) == G_TYPE_NONE)
+    {
+        return TRUE;
+    }
+
+    switch(G_VALUE_TYPE(value1)) {
+#define HANDLE_CMP_GV(TYPE, getter) \
+        case G_TYPE_ ## TYPE: \
+            return g_value_get_ ## getter(value1) == g_value_get_ ## getter(value2)
+
+        HANDLE_CMP_GV(CHAR, char);
+        HANDLE_CMP_GV(UCHAR, uchar);
+        HANDLE_CMP_GV(BOOLEAN, boolean);
+        HANDLE_CMP_GV(INT, int);
+        HANDLE_CMP_GV(UINT, uint);
+        HANDLE_CMP_GV(INT64, int64);
+        HANDLE_CMP_GV(UINT64, uint64);
+        HANDLE_CMP_GV(FLOAT, float);
+        HANDLE_CMP_GV(DOUBLE, double);
+
+        case G_TYPE_STRING:
+            return !g_strcmp0(g_value_get_string(value1), g_value_get_string(value2));
+
+        default:
+            if(G_VALUE_TYPE(value1) == XFCONF_TYPE_INT16)
+                return xfconf_g_value_get_int16(value1) == xfconf_g_value_get_uint16(value2);
+            else if(G_VALUE_TYPE(value1) == XFCONF_TYPE_UINT16)
+                return xfconf_g_value_get_uint16(value1) == xfconf_g_value_get_uint16(value2);
+            break;
+#undef HANDLE_CMP_GV
+    }
+
+    return FALSE;
+}
+
 void
 _xfconf_gvalue_free(GValue *value)
 {
