@@ -32,10 +32,12 @@
 #include "xfconf-gvaluefuncs.h"
 #include "xfconf-private.h"
 #include "xfconf-marshal.h"
-//#include "xfconf-types.h"
-//#include "xfconf-common-private.h"
-//#include "xfconf.h"
-//#include "xfconf-alias.h"
+#if 0
+#include "xfconf-types.h"
+#include "xfconf-common-private.h"
+#include "xfconf.h"
+#include "xfconf-alias.h"
+#endif
 
 #define DEFAULT_MAX_ENTRIES  -1  /* no limit */
 #define DEFAULT_MAX_AGE      (60*60)  /* 1 hour */
@@ -489,13 +491,17 @@ xfconf_cache_set_property_reply_handler(DBusGProxy *proxy,
 
     old_item = g_hash_table_lookup(cache->pending_calls, call);
     if(G_UNLIKELY(!old_item)) {
+#ifndef NDEBUG
         g_debug("Couldn't find old cache item based on pending call (libxfconf bug?)");
+#endif
         goto out;
     }
 
     item = g_tree_lookup(cache->properties, old_item->property);
     if(G_UNLIKELY(!item)) {
+#ifndef NDEBUG
         g_debug("Couldn't find current cache item based on pending call (libxfconf bug?)");
+#endif
         goto out;
     }
 
