@@ -1,7 +1,7 @@
 /*
  *  xfconf
  *
- *  Copyright (c) 2007 Brian Tarricone <bjt23@cornell.edu>
+ *  Copyright (c) 2007-2009 Brian Tarricone <bjt23@cornell.edu>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -69,7 +69,7 @@
 #define CONFIG_DIR_STEM  "xfce4/xfconf/" XFCONF_BACKEND_PERCHANNEL_XML_TYPE_ID "/"
 #define CONFIG_FILE_FMT  CONFIG_DIR_STEM "%s.xml"
 #define CACHE_TIMEOUT    (20*60*1000)  /* 20 minutes */
-#define WRITE_TIMEOUT    (5*1000)  /* 5 seconds */
+#define WRITE_TIMEOUT    (5)  /* 5 seconds */
 #define MAX_PROP_PATH    (4096)
 
 struct _XfconfBackendPerchannelXml
@@ -130,9 +130,6 @@ typedef struct
     GValue *list_value;
 } XmlParserState;
 
-static void xfconf_backend_perchannel_xml_class_init(XfconfBackendPerchannelXmlClass *klass);
-
-static void xfconf_backend_perchannel_xml_init(XfconfBackendPerchannelXml *instance);
 static void xfconf_backend_perchannel_xml_finalize(GObject *obj);
 
 static void xfconf_backend_perchannel_xml_backend_init(XfconfBackendInterface *iface);
@@ -280,7 +277,8 @@ xfconf_backend_perchannel_xml_initialize(XfconfBackend *backend,
         if(error) {
             g_set_error(error, XFCONF_ERROR,
                         XFCONF_ERROR_WRITE_FAILURE,
-                        _("Unable to create configuration directory"));
+                        _("Unable to create configuration directory \"%s\""),
+                        path);
         }
         g_free(path);
         return FALSE;
@@ -327,7 +325,7 @@ xfconf_backend_perchannel_xml_set(XfconfBackend *backend,
             if(error) {
                 g_set_error(error, XFCONF_ERROR,
                             XFCONF_ERROR_PERMISSION_DENIED,
-                            _("You don't have permission to modify property \"%s\" on channel \"%s\""),
+                            _("Permission denied while modifying property \"%s\" on channel \"%s\""),
                             property, channel_name);
             }
             return FALSE;
@@ -544,7 +542,7 @@ nodes_do_prop_reset(GNode *node,
 {
     PropChangeData *pdata = data;
     XfconfProperty *prop = node->data;
-    gchar prop_fullname[4096];
+    gchar prop_fullname[MAX_PROP_PATH];
 
 
     /* we don't signal if |value| isn't set but |system_value| is,
@@ -564,6 +562,24 @@ nodes_do_prop_reset(GNode *node,
     return FALSE;
 }
 
+static gboolean
+nodes_clean_up(GNode *node,
+               gpointer data)
+{
+    XfconfProperty *prop = node->data;
+
+    /* clean up dangling nodes in tree without system defaults */
+    if(!node->children
+       && !G_VALUE_TYPE(&prop->value)
+       && !G_VALUE_TYPE(&prop->system_value)
+       && !prop->locked) {
+        g_node_unlink(node);
+        xfconf_proptree_destroy(node);
+    }
+
+    return FALSE;
+}
+
 static gboolean
 do_reset_channel(XfconfBackend *backend,
                  const gchar *channel_name,
@@ -666,7 +682,9 @@ xfconf_backend_perchannel_xml_reset(XfconfBackend *backend,
             g_node_traverse(top, G_POST_ORDER, G_TRAVERSE_ALL, -1,
                             nodes_do_prop_reset, &pdata);
 
-            /* FIXME: clean up dangling nodes in tree without system defaults */
+            /* clean up dangling nodes in tree without system defaults */
+            g_node_traverse(top, G_POST_ORDER, G_TRAVERSE_ALL, -1,
+                            nodes_clean_up, NULL);
         } else {
             /* remove the entire channel */
             return do_reset_channel(backend, channel_name,
@@ -996,9 +1014,9 @@ xfconf_backend_perchannel_xml_schedule_save(XfconfBackendPerchannelXml *xbpx,
     if(xbpx->save_id)
         g_source_remove(xbpx->save_id);
 
-    xbpx->save_id = g_timeout_add(WRITE_TIMEOUT,
-                                  xfconf_backend_perchannel_xml_save_timeout,
-                                  xbpx);
+    xbpx->save_id = g_timeout_add_seconds(WRITE_TIMEOUT,
+                                          xfconf_backend_perchannel_xml_save_timeout,
+                                          xbpx);
 }
 
 static XfconfChannel *
@@ -1066,7 +1084,7 @@ xfconf_xml_handle_channel(XmlParserState *state,
 
     /* compare versions */
     p = strstr(version, ".");
-    maj_ver_len = p ? p - version : strlen(version);
+    maj_ver_len = p ? (gsize)(p - version) : strlen(version);
     if(maj_ver_len != strlen(FILE_VERSION_MAJOR)
        || strncmp(version, FILE_VERSION_MAJOR, maj_ver_len))
     {
@@ -1646,7 +1664,7 @@ xfconf_backend_perchannel_xml_load_channel(XfconfBackendPerchannelXml *xbpx,
 {
     XfconfChannel *channel = NULL;
     gchar *filename_stem, **filenames, *user_file;
-    gint i;
+    gint i, length;
     XfconfProperty *prop;
 
     TRACE("entering");
@@ -1671,9 +1689,11 @@ xfconf_backend_perchannel_xml_load_channel(XfconfBackendPerchannelXml *xbpx,
     prop->name = g_strdup("/");
     channel->properties = g_node_new(prop);
 
-    /* read in system files */
-    for(i = 0; filenames[i]; ++i) {
-        if(user_file && !strcmp(filenames[i], user_file))
+    /* read in system files, we do this in reversed order to properly 
+     * follow the xdg spec, see bug #6079 for more information */
+    length = g_strv_length(filenames);
+    for(i = length - 1; i >= 0; --i) {
+        if(!g_strcmp0(user_file, filenames[i]))
             continue;
         xfconf_backend_perchannel_xml_merge_file(xbpx, filenames[i], TRUE,
                                                  channel, NULL);
@@ -1936,6 +1956,19 @@ xfconf_backend_perchannel_xml_flush_channel(XfconfBackendPerchannelXml *xbpx,
     if(fputs("</channel>\n", fp) == EOF)
         goto out;
 
+    if(fflush(fp))
+        goto out;
+
+#if defined(HAVE_FDATASYNC)
+    if(fdatasync(fileno(fp)))
+        goto out;
+#elif defined(HAVE_FSYNC)
+    if(fsync(fileno(fp)))
+        goto out;
+#else
+    sync();
+#endif
+
     if(fclose(fp)) {
         fp = NULL;
         goto out;
